@@ -27,6 +27,7 @@
 
 #include <QAbstractAnimation>
 #include <QGraphicsOpacityEffect>
+#include <QStringBuilder>
 
 #include <Transaction>
 
@@ -86,7 +87,7 @@ UpdateDetails::~UpdateDetails()
 {
 }
 
-void UpdateDetails::setPackage(const QString &packageId, Package::Info updateInfo)
+void UpdateDetails::setPackage(const QString &packageId, Transaction::Info updateInfo)
 {
     if (m_packageId == packageId) {
         return;
@@ -96,19 +97,18 @@ void UpdateDetails::setPackage(const QString &packageId, Package::Info updateInf
     m_updateInfo = updateInfo;
     m_currentDescription.clear();
     if (m_transaction) {
-        disconnect(m_transaction, SIGNAL(package(PackageKit::Package)),
-                   this, SLOT(updateDetail(PackageKit::Package)));
+        disconnect(m_transaction, SIGNAL(updateDetail(QString,QStringList,QStringList,QStringList,QStringList,QStringList,PackageKit::Transaction::Restart,QString,QString,PackageKit::Transaction::UpdateState,QDateTime,QDateTime)),
+                   this, SLOT(updateDetail(QString,QStringList,QStringList,QStringList,QStringList,QStringList,PackageKit::Transaction::Restart,QString,QString,PackageKit::Transaction::UpdateState,QDateTime,QDateTime)));
         disconnect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
                    this, SLOT(display()));
     }
 
-    Package package(m_packageId, Package::UnknownInfo, QString());
     m_transaction = new Transaction(this);
-    connect(m_transaction, SIGNAL(package(PackageKit::Package)),
-            this, SLOT(updateDetail(PackageKit::Package)));
+    connect(m_transaction, SIGNAL(updateDetail(QString,QStringList,QStringList,QStringList,QStringList,QStringList,PackageKit::Transaction::Restart,QString,QString,PackageKit::Transaction::UpdateState,QDateTime,QDateTime)),
+            this, SLOT(updateDetail(QString,QStringList,QStringList,QStringList,QStringList,QStringList,PackageKit::Transaction::Restart,QString,QString,PackageKit::Transaction::UpdateState,QDateTime,QDateTime)));
     connect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
             this, SLOT(display()));
-    m_transaction->getUpdateDetail(package);
+    m_transaction->getUpdateDetail(m_packageId);
     Transaction::InternalError error = m_transaction->error();
     if (error) {
         disconnect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
@@ -165,170 +165,171 @@ void UpdateDetails::display()
     }
 }
 
-void UpdateDetails::updateDetail(const PackageKit::Package &package)
+void UpdateDetails::updateDetail(const QString &packageID,
+                                 const QStringList &updates,
+                                 const QStringList &obsoletes,
+                                 const QStringList &vendorUrls,
+                                 const QStringList &bugzillaUrls,
+                                 const QStringList &cveUrls,
+                                 PackageKit::Transaction::Restart restart,
+                                 const QString &updateText,
+                                 const QString &changelog,
+                                 PackageKit::Transaction::UpdateState state,
+                                 const QDateTime &issued,
+                                 const QDateTime &updated)
 {
     //format and show description
     QString description;
 
     // update type (ie Security Update)
-    if (m_updateInfo == Package::InfoEnhancement) {
+    if (m_updateInfo == Transaction::InfoEnhancement) {
         description += "<p>" +
                        i18n("This update will add new features and expand functionality.") +
                        "</p>";
-    } else if (m_updateInfo == Package::InfoBugfix) {
+    } else if (m_updateInfo == Transaction::InfoBugfix) {
         description += "<p>" +
                        i18n("This update will fix bugs and other non-critical problems.") +
                        "</p>";
-    } else if (m_updateInfo == Package::InfoImportant) {
+    } else if (m_updateInfo == Transaction::InfoImportant) {
         description += "<p>" +
                        i18n("This update is important as it may solve critical problems.") +
                        "</p>";
-    } else if (m_updateInfo == Package::InfoSecurity) {
+    } else if (m_updateInfo == Transaction::InfoSecurity) {
         description += "<p>" +
                        i18n("This update is needed to fix a security vulnerability with this package.") +
                        "</p>";
-    } else if (m_updateInfo == Package::InfoBlocked) {
+    } else if (m_updateInfo == Transaction::InfoBlocked) {
         description += "<p>" +
                        i18n("This update is blocked.") +
                        "</p>";
     }
 
     // Issued and Updated
-    if (!package.issued().toString().isEmpty() && !package.updated().toString().isEmpty()) {
+    if (!issued.isNull() && !updated.isNull()) {
         description += "<p>" +
                        i18n("This notification was issued on %1 and last updated on %2.",
-                            KGlobal::locale()->formatDateTime(package.issued(), KLocale::ShortDate),
-                            KGlobal::locale()->formatDateTime(package.updated(), KLocale::ShortDate)) +
+                            KGlobal::locale()->formatDateTime(issued, KLocale::ShortDate),
+                            KGlobal::locale()->formatDateTime(updated, KLocale::ShortDate)) +
                        "</p>";
-    } else if (!package.issued().toString().isEmpty()) {
+    } else if (!issued.isNull()) {
         description += "<p>" +
                        i18n("This notification was issued on %1.",
-                            KGlobal::locale()->formatDateTime(package.issued(), KLocale::ShortDate)) +
+                            KGlobal::locale()->formatDateTime(issued, KLocale::ShortDate)) +
                        "</p>";
     }
 
     // Description
-    if (!package.updateText().isEmpty()) {
-        QString updateText = package.updateText();
-        updateText.replace('\n', "<br/>");
-        updateText.replace(' ', "&nbsp;");
+    if (!updateText.isEmpty()) {
+        QString _updateText = updateText;
+        _updateText.replace('\n', "<br/>");
+        _updateText.replace(' ', "&nbsp;");
         description += "<p>" +
-                       updateText +
+                       _updateText +
                        "</p>";
     }
 
     // links
     //  Vendor
-    if (!package.vendorUrl().isEmpty()) {
+    if (!vendorUrls.isEmpty()) {
         description += "<p>" +
                        i18np("For more information about this update please visit this website:",
                              "For more information about this update please visit these websites:",
-                             package.vendorUrl().split(';').size() % 2) + "<br/>" +
-                       getLinkList(package.vendorUrl()) +
+                             vendorUrls.size()) + "<br/>" +
+                       getLinkList(vendorUrls) +
                        "</p>";
     }
 
     //  Bugzilla
-    if (!package.bugzillaUrl().isEmpty()) {
+    if (!bugzillaUrls.isEmpty()) {
         description += "<p>" +
                        i18np("For more information about bugs fixed by this update please visit this website:",
                              "For more information about bugs fixed by this update please visit these websites:",
-                             package.bugzillaUrl().split(';').size() % 2) + "<br/>" +
-                       getLinkList(package.bugzillaUrl()) +
+                             bugzillaUrls.size()) + "<br/>" +
+                       getLinkList(bugzillaUrls) +
                        "</p>";
     }
 
     //  CVE
-    if (!package.cveUrl().isEmpty()) {
+    if (!cveUrls.isEmpty()) {
         description += "<p>" +
                        i18np("For more information about this security update please visit this website:",
                              "For more information about this security update please visit these websites:",
-                             package.cveUrl().split(';').size() % 2) + "<br/>" +
-                       getLinkList(package.cveUrl()) +
+                             cveUrls.size()) + "<br/>" +
+                       getLinkList(cveUrls) +
                        "</p>";
     }
 
     // Notice (about the need for a reboot)
-    if (package.restart() == Package::RestartSystem) {
+    if (restart == Transaction::RestartSystem) {
         description += "<p>" +
                        i18n("The computer will have to be restarted after the update for the changes to take effect.") +
                        "</p>";
-    } else if (package.restart() == Package::RestartSession) {
+    } else if (restart == Transaction::RestartSession) {
         description += "<p>" +
                        i18n("You will need to log out and back in after the update for the changes to take effect.") +
                        "</p>";
     }
 
     // State
-    if (package.state() == Package::UpdateStateUnstable) {
+    if (state == Transaction::UpdateStateUnstable) {
         description += "<p>" +
                        i18n("The classification of this update is unstable which means it is not designed for production use.") +
                        "</p>";
-    } else if (package.state() == Package::UpdateStateTesting) {
+    } else if (state == Transaction::UpdateStateTesting) {
         description += "<p>" +
                        i18n("This is a test update, and is not designed for normal use. Please report any problems or regressions you encounter.") +
                        "</p>";
     }
 
     // only show changelog if we didn't have any update text
-    if (package.updateText().isEmpty() && !package.changelog().isEmpty()) {
-        QString changelog = package.changelog();
-        changelog.replace('\n', "<br/>");
-        changelog.replace(' ', "&nbsp;");
+    if (updateText.isEmpty() && !changelog.isEmpty()) {
+        QString _changelog = changelog;
+        _changelog.replace('\n', "<br/>");
+        _changelog.replace(' ', "&nbsp;");
         description += "<p>" +
                        i18n("The developer logs will be shown as no description is available for this update:") +
                        "<br/>" +
-                       changelog +
+                       _changelog +
                        "</p>";
     }
 
     // Updates (lists of packages that are updated)
-    if (package.updates().size()) {
+    if (!updates.isEmpty()) {
         description += "<p>" + i18n("Updates:") + "<br/>";
-        QStringList updates;
-        foreach (const Package &p, package.updates()) {
-             updates += QString::fromUtf8("\xE2\x80\xA2 ") + p.name() + " - " + p.version();
+        QStringList _updates;
+        foreach (const QString &pid, updates) {
+             _updates += QString::fromUtf8("\xE2\x80\xA2 ") + Transaction::packageName(pid) + " - " + Transaction::packageVersion(pid);
         }
-        description += updates.join("<br/>") + "</p>";
+        description += _updates.join("<br/>") + "</p>";
     }
 
     // Obsoletes (lists of packages that are obsoleted)
-    if (package.obsoletes().size()) {
+    if (obsoletes.size()) {
         description += "<p></b>" + i18n("Obsoletes:") + "</b><br/>";
-        QStringList obsoletes;
-        foreach (const Package &p, package.obsoletes()) {
-             obsoletes += QString::fromUtf8("\xE2\x80\xA2 ") + p.name() + " - " + p.version();
+        QStringList _obsoletes;
+        foreach (const QString &pid, obsoletes) {
+             _obsoletes += QString::fromUtf8("\xE2\x80\xA2 ") + Transaction::packageName(pid) + " - " + Transaction::packageVersion(pid);
         }
-        description += obsoletes.join("<br>/") + "</p>";
+        description += _obsoletes.join("<br>/") + "</p>";
     }
 
     // Repository (this is the repository the package comes from)
-    if (!package.data().isEmpty()) {
-         description += "<p>" + i18n("Repository:") + ' ' + package.data() + "</p>";
+    if (!Transaction::packageData(packageID).isEmpty()) {
+         description += "<p>" + i18n("Repository: %1", Transaction::packageData(packageID)) + "</p>";
     }
 
     m_currentDescription = description;
     m_busySeq->stop();
 }
 
-QString UpdateDetails::getLinkList(const QString &links) const
+QString UpdateDetails::getLinkList(const QStringList &urls) const
 {
-    QStringList linkList = links.split(';');
-    int length = linkList.size();
     QString ret;
-
-    // check for malformed strings with ';'
-    if (length % 2 != 0) {
-        kWarning() << "length not correct, correcting";
-        length--;
-    }
-
-    for (int i = 0; i < length; i += 2) {
+    foreach (const QString &url, urls) {
         if (!ret.isEmpty()) {
             ret += "<br/>";
         }
-        ret += QString::fromUtf8(" \xE2\x80\xA2 <a href=\"") + linkList.at(i) + "\">"
-               + linkList.at(i + 1) + "</a>";
+        ret += QString::fromUtf8(" \xE2\x80\xA2 <a href=\"") % url % QLatin1String("\">") % url % QLatin1String("</a>");
     }
     return ret;
 }
