@@ -22,6 +22,7 @@
 #include "ApperdThread.h"
 
 #include <KGenericFactory>
+#include <KDebug>
 
 K_PLUGIN_FACTORY(ApperFactory, registerPlugin<ApperD>();)
 K_EXPORT_PLUGIN(ApperFactory("apperd"))
@@ -29,12 +30,22 @@ K_EXPORT_PLUGIN(ApperFactory("apperd"))
 ApperD::ApperD(QObject *parent, const QList<QVariant> &) :
     KDEDModule(parent)
 {
+//    m_thread = new QThread(this);
     m_apperThread = new ApperdThread;
+//    m_apperThread->moveToThread(m_thread);
+//    m_thread->start();
+
+    // Make all our init code run on the thread since
+    // the DBus calls were made blocking
+    QTimer::singleShot(0, m_apperThread, SLOT(init()));
 }
 
 ApperD::~ApperD()
 {
     // delete the apper thread code, don't use delete later since it has moved
     // to the stopped thread
-    delete m_apperThread;
+    m_apperThread->deleteLater();
+//    m_thread->quit();
+//    m_thread->wait();
+//    delete m_thread;
 }
